@@ -21,7 +21,7 @@ OUT_NODE1=$(mktemp /tmp/node1.XXXXXX)
 PORT_NODE1=3000
 #DB1="-W /tmp/node1 -D sled"
 #DB1="-W /tmp/node1 -D sneakers"
-$DIR/../target/debug/dtnd -d -j0 -i0 -w $PORT_NODE1 -C mtcp:2342 -e incoming -r epidemic -n node1 -s mtcp://127.0.0.1:4223/node2 $DB1 2>&1 &> $OUT_NODE1 &
+$DIR/../target/$TARGET/dtnd -d -j5s -i0 -w $PORT_NODE1 -C mtcp:2342 -e incoming -r epidemic -n node1 -s mtcp://127.0.0.1:4223/node2 $DB1 2>&1 &> $OUT_NODE1 &
 PID_NODE1=$!
 echo node1 pid: $PID_NODE1
 echo node1 out: $OUT_NODE1
@@ -32,7 +32,7 @@ OUT_NODE2=$(mktemp /tmp/node2.XXXXXX)
 PORT_NODE2=3001
 #DB2="-W /tmp/node2 -D sled"
 #DB2="-W /tmp/node2 -D sneakers"
-$DIR/../target/debug/dtnd -d -j0 -i0 -w $PORT_NODE2 -C mtcp:4223 -e incoming -r epidemic \
+$DIR/../target/$TARGET/dtnd -d -j5s -i0 -w $PORT_NODE2 -C mtcp:4223 -e incoming -r epidemic \
     -n node2 \
     -s mtcp://127.0.0.1:2342/node1 \
     -s mtcp://127.0.0.1:2432/node3 \
@@ -46,7 +46,7 @@ OUT_NODE3=$(mktemp /tmp/node3.XXXXXX)
 PORT_NODE3=3002
 #DB3="-W /tmp/node3 -D sled"
 #DB3="-W /tmp/node3 -D sneakers"
-$DIR/../target/debug/dtnd -d -j0 -i0 -w $PORT_NODE3 -C mtcp:2432 -e incoming -r epidemic -n node3 -s mtcp://127.0.0.1:4223/node2 $DB3 2>&1 &> $OUT_NODE3 &
+$DIR/../target/$TARGET/dtnd -d -j5s -i0 -w $PORT_NODE3 -C mtcp:2432 -e incoming -r epidemic -n node3 -s mtcp://127.0.0.1:4223/node2 $DB3 2>&1 &> $OUT_NODE3 &
 PID_NODE3=$!
 echo node3 pid: $PID_NODE3
 echo node3 out: $OUT_NODE3
@@ -57,12 +57,12 @@ sleep 1
 echo
 
 echo "Sending 'test' to node 3"
-echo test | $DIR/../target/debug/dtnsend -r dtn://node3/incoming -p $PORT_NODE1
+echo test | $DIR/../target/$TARGET/dtnsend -r dtn://node3/incoming -p $PORT_NODE1
 
 sleep 1
 
 echo -n "Receiving on node 3: "
-$DIR/../target/debug/dtnrecv -v -e incoming -p $PORT_NODE3
+$DIR/../target/$TARGET/dtnrecv -v -e incoming -p $PORT_NODE3
 RC=$? 
 echo "RET: $RC" 
 echo 
